// Copyright (C) 2012-2014 ChaosForge Ltd
// http://chaosforge.org/
//
// This file is part of NV Libraries.
// For conditions of distribution and use, see copyright notice in nv.hh

/**
 * @file md3_loader.hh
 * @author Kornel Kisielewicz
 * @brief md3 loader
 */

#ifndef NV_MD3_LOADER_HH
#define NV_MD3_LOADER_HH

#include <nv/core/common.hh>
#include <unordered_map>
#include <nv/stl/array.hh>
#include <nv/core/transform.hh>
#include <nv/interface/mesh_data.hh>
#include <nv/interface/mesh_loader.hh>

namespace nv 
{
	struct md3_key
	{
		transform tform;
	};


	class md3_loader : public mesh_loader
	{
	public:
		md3_loader( bool merge_all = true );
		virtual ~md3_loader();
		virtual bool load( stream& source );
		virtual mesh_data* release_mesh_data( size_t index = 0 );
		virtual mesh_data_pack* release_mesh_data_pack();
		virtual size_t get_mesh_count() const { return 1; }
		size_t get_max_frames() const;
		virtual size_t get_nodes_data_count() const { return 1; }
		virtual mesh_nodes_data* release_mesh_nodes_data( size_t = 0 );
	private:
		void release_mesh_frame( mesh_data* data, sint32 frame, sint32 surface );
		key_raw_channel* load_tags( const string_ref& tag );
		bool m_merge_all;
		void* m_md3;
	};

}

#endif // NV_OBJ_LOADER_HH
